@@ -11,7 +11,8 @@
 
 from __future__ import absolute_import
 
-__version__ = '0.8.0'
+__version__ = '0.8.1'
 
-from preggy.core import (assertion, create_assertions, fix_string, Expect as expect)
+from preggy.core import (assertion, create_assertions, Expect as expect)
 from preggy.assertions import *
+
